/* === This file is part of Calamares - <http://github.com/calamares> ===
 *
 *   Copyright 2019, Adriaan de Groot <groot@kde.org>
 *
 *   Calamares is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Calamares is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Calamares. If not, see <http://www.gnu.org/licenses/>.
 */

#include "RequirementsChecker.h"

#include "Module.h"
#include "Requirement.h"

#include "utils/Logger.h"

#include <algorithm>

#include <QtConcurrent/QtConcurrent>
#include <QFuture>
#include <QFutureWatcher>
#include <QTimer>


namespace Calamares
{

void
check( Module * const &m, RequirementsChecker *c )
{
    RequirementsList l = m->checkRequirements();
    if ( l.count() > 0 )
        c->addCheckedRequirements( l );
    c->requirementsProgress( QObject::tr( "Requirements checking for module %1 is complete." ).arg( m->name() ) );
}

RequirementsChecker::RequirementsChecker( QVector< Module* > modules, QObject* parent )
    : QObject( parent )
    , m_modules( std::move( modules ) )
{
    m_watchers.reserve( m_modules.count() );
    m_collectedRequirements.reserve( m_modules.count() );
}

RequirementsChecker::~RequirementsChecker()
{
}

void
RequirementsChecker::run()
{
    for (const auto& module : m_modules )
    {
        Watcher *watcher = new Watcher( this );
        watcher->setFuture( QtConcurrent::run( check, module, this ) );
        m_watchers.append( watcher );
        connect( watcher, &Watcher::finished, this, &RequirementsChecker::finished );
    }
}

void
RequirementsChecker::finished()
{
    if ( std::all_of( m_watchers.cbegin(), m_watchers.cend(), []( const Watcher *w ) { return w && w->isFinished(); } ) )
    {
        cDebug() << "All requirements have been checked.";

        bool acceptable = true;
        int count = 0;
        for ( const auto& r : m_collectedRequirements )
        {
            if ( r.mandatory && !r.satisfied )
            {
                cDebug() << "  .. requirement" << count << r.name << "is not satisfied.";
                acceptable = false;
            }
            ++count;
        }

        emit requirementsComplete( acceptable );
        QTimer::singleShot(0, this, &RequirementsChecker::done );
    }
}

void
RequirementsChecker::addCheckedRequirements( RequirementsList l )
{
    static QMutex addMutex;
    {
        QMutexLocker lock( &addMutex );
        m_collectedRequirements.append( l );
    }
    cDebug() << "Added" << l.count() << "requirement results";
    emit requirementsResult( l );
}

}
